// Windows-specific token interactivity.
// As well as shield keeping between levels
// and special stages.

sfxinfo[sfx_cdfm67] = {
	caption = "Ring Spill"
}

sfxinfo[sfx_cdfm73] = {
	caption = "Perfect!"
}

local function controlpanel()
	S_StartSound(nil, sfx_uihmsn, consoleplayer)
end

local netframework = CV_RegisterVar({ // Compromise with having default special stages.
	name = "TokenCompile",
	defaultvalue = 0,
	flags = CV_NETVAR | CV_SHOWMODIF,
	PossibleValue = {default=0, legacy=1},
	func = controlpanel
})

local function Token2GemSFX()
	S_StartSound(nil, sfx_cgot)
	S_StartSound(nil, sfx_chchng)
end

local function TokenWindowsReward()
	for player in players.iterate
		P_GivePlayerRings(player, 50) // Rings, people, Rings!
		S_StartSound(player.mo, sfx_cdfm67, player)
	end
end

// The old special stages continue Lua.
// Script by SteelT.
local therings = 0
local thesphere = 0
local collectedspheres = 0
local hooray = false

addHook("MapLoad", function(gamemap)
	therings = 0
	thesphere = 0
	collectedspheres = 0
	hooray = false
	if (multiplayer) then return end
	if (maptol & TOL_NIGHTS) then return end
	if (G_IsSpecialStage(gamemap)) then
		for item in mobjs.iterate() do
			if (item.type == MT_RING) or (item.type == MT_BLUESPHERE) then
				if (item.type == MT_RING) then therings = $ + 1 end
				if (item.type == MT_BLUESPHERE) then thesphere = $ + 1 end
			end
		end
	end
end)

addHook("PlayerThink", function(player)
	if (multiplayer) then return end
	if (maptol & TOL_NIGHTS) then return end
	if (G_IsSpecialStage(gamemap)) then
		if (therings > 0 and player.rings >= therings/2) and not player.gotcontinue then
			player.gotcontinue = 1
			player.continues = $ + 1
			S_StartSound(nil, sfx_s3kac, player)
		end
		if thesphere > 0 and (collectedspheres >= thesphere - 2) and not hooray then
			S_StartSound(nil, sfx_cdfm73, player)
			player.rings = $ + 1000
			hooray = true
			CONS_Printf(player, "Sphere Mania!")
			player.thesphere = true
		end
		if player.exiting > 0 then
			if hooray then
				if leveltime%5 == 1 then
					local glow = P_SpawnGhostMobj(player.mo)
					glow.destscale = player.mo.scale*32
					glow.flags = $ & ~MF_NOCLIPHEIGHT
				end
			else
				player.thesphere = false
			end
		end
	else
		player.thesphere = false
	end
end)

addHook("TouchSpecial", function(special, entity)
	for player in players.iterate()
		if player and player.valid then
			if collectedspheres < thesphere then
				collectedspheres = $ + 1
			end
		end
	end
end, MT_BLUESPHERE)

addHook("IntermissionThinker", function()
	if collectedspheres > 0 then collectedspheres = 0 end
end)

// From KeepShield V3 by Zippy_Zolton.
// Be smart with which shield you finish a level with.
// Might help you or end up being useless.
addHook("PlayerThink", function(player)
	if (player.mo and player.mo.valid) and player.mo.skin != "windows" then return
	elseif (player.mo and player.mo.valid) and player.mo.skin == "windows" then
		if not (player and player.mo and player.mo.valid and player.exiting and (gametyperules & GTR_FRIENDLY)) return end
		player.shieldtransfer = player.powers[pw_shield]
	end
end)

addHook("PlayerSpawn", function(player)
	if (player.mo and player.mo.valid) then
		if player.shieldtransfer == nil or not (gametyperules & GTR_FRIENDLY) then return end
		if player.mo.skin != "windows" then return
		elseif player.mo.skin == "windows" then
			player.powers[pw_shield] = player.shieldtransfer
			P_SpawnShieldOrb(player)
			player.shieldtransfer = nil
		end
	end
end)

// Actual token shenanigans.
addHook("TouchSpecial", function(special, grabber)
	if (grabber and grabber.valid) and grabber.skin != "windows" then return
	elseif (grabber and grabber.valid) and grabber.skin == "windows" then
		if netframework.value == 1 then
			if emeralds & EMERALD1 == 0 then
				emeralds = $ | EMERALD1
				Token2GemSFX()
				P_AddPlayerScore(grabber.player, 2500)
			elseif emeralds & EMERALD2 == 0 then
				emeralds = $ | EMERALD2
				Token2GemSFX()
				P_AddPlayerScore(grabber.player, 2500)
			elseif emeralds & EMERALD3 == 0 then
				emeralds = $ | EMERALD3
				Token2GemSFX()
				P_AddPlayerScore(grabber.player, 2500)
			elseif emeralds & EMERALD4 == 0 then
				emeralds = $ | EMERALD4
				Token2GemSFX()
				P_AddPlayerScore(grabber.player, 2500)
			elseif emeralds & EMERALD5 == 0 then
				emeralds = $ | EMERALD5
				Token2GemSFX()
				P_AddPlayerScore(grabber.player, 2500)
			elseif emeralds & EMERALD6 == 0 then
				emeralds = $ | EMERALD6
				Token2GemSFX()
				P_AddPlayerScore(grabber.player, 2500)
			elseif emeralds & EMERALD7 == 0 then
				emeralds = $ | EMERALD7
				Token2GemSFX()
				P_AddPlayerScore(grabber.player, 2500)
			else TokenWindowsReward() end
		else
			if not All7Emeralds(emeralds) then
				token = $ + 1
				S_StartSound(grabber, sfx_zelda)
			else
				TokenWindowsReward()
			end
			P_AddPlayerScore(grabber.player, 2500)
			S_StartSound(grabber, sfx_chchng)
		end
		P_KillMobj(special)
		return true
	end
end, MT_TOKEN)