// Custom Lua for File Toss.
// Hello, Mario.

freeslot("SFX_FILETH")

sfxinfo[sfx_fileth] = {
        singular = false,
        priority = 42,
        flags = SF_X2AWAYSOUND,
		caption = "Recycle Bin"
}

local defbind = true

local function controlpanel()
	S_StartSound(nil, sfx_uihmsn, consoleplayer)
end

// Faster file access... without TRIM.
// Enable at your own risk!

local intenserecycle = CV_RegisterVar({ 
	name = "ssd",
	defaultvalue = 0,
	flags = CV_NETVAR | CV_SHOWMODIF | CV_CALL,
	PossibleValue = {notpresent=0, installed=1},
	func = controlpanel
})

// Let's give the player option to change the File Toss use key.
local function filetosschange(player, arg)
	if arg then
		arg = arg:lower()
		if arg == "0" or arg == "spin" or arg == "false"
			player.ftdefbind = false
			CONS_Printf(player, "File Toss is activated with Spin instead.")
			S_StartSound(nil, sfx_uihmsn, player)
			defbind = false
		elseif arg == "1" or arg == "ringtoss" or arg == "true" or arg == "default"
			player.ftdefbind = true
			CONS_Printf(player, "File Toss is activated with Ring Toss.")
			S_StartSound(nil, sfx_uihmsn, player)
			defbind = true
		else
			CONS_Printf(player, "Ring Toss is used to activate File Toss. If false, Spin is used instead. Currently " + player.ftdefbind + ".")
		end
	end
end

// Actual command to kick off the aforementioned change.
COM_AddCommand("filetossbind", function(player, arg)
	if arg then filetosschange(player, arg)
	else CONS_Printf(player, "Ring Toss is used to activate File Toss. If false, Spin is used instead. Currently " + player.ftdefbind + ".")
	end
end)

addHook("PlayerSpawn", function(player)
	if not (player.mo and player.mo.valid) then return end
	if player.ftdefbind == nil then player.ftdefbind = defbind
	else
		if netgame then
			if player.ftdefbind and player.mo.skin == "windows" then
				CONS_Printf(player, "Ring Toss is used for File Toss.")
			elseif (not player.ftdefbind) and player.mo.skin == "windows" then
				CONS_Printf(player, "Spin is used for File Toss.")
			end
		end
	end
end)

// Actual File Toss thing.
addHook("PlayerThink", function(player)
	if (player.mo and player.mo.valid) and player.mo.skin == "windows" then
		if player.baudrate == nil then player.baudrate = 0 end
		if player.baudrate < 19 and gamestate != 4 then
			player.baudrate = $ + 1
		end
		player.targetfile = P_LookForEnemies(player, false, true)
		if (player.cmd.buttons & BT_ATTACK and player.ftdefbind)
		or (player.cmd.buttons & BT_SPIN and not player.ftdefbind) then
			player.filetossfire = true
		else
			player.filetossfire = false
		end
		if (player.targetfile and player.targetfile.valid and player.targetfile.health) then
			player.weapondelay = 1
			if player.mo.state == S_PLAY_WALK or player.mo.state == S_PLAY_RUN or (player.pflags & PF_JUMPED) or player.mo.state == S_PLAY_STND or player.mo.state == S_PLAY_WAIT then
				P_SpawnLockOn(player, player.targetfile, S_LOCKON1)
			end
			if player.filetossfire then
				if player.baudrate >= 19 and P_PlayerInPain(player) == false and player.playerstate != PST_DEAD and player.exiting == 0 then
					if player.targetfile and player.targetfile.valid then
						local file = P_SpawnMissile(player.mo, player.targetfile, MT_WINFILE)
						if player.powers[pw_super] != 0 then
							if player.hyper and player.hyper.capable then
								player.baudrate = 18
								file.momx = $*3/2
								file.momy = $*3/2
								file.momz = $*3/2
							elseif intenserecycle.value == 1 then
								player.baudrate = 18
								file.momx = $*4/3
								file.momy = $*4/3
								file.momz = $*4/3
							else
								player.baudrate = 14
								file.momx = $*4/3
								file.momy = $*4/3
								file.momz = $*4/3
							end
						else
							if intenserecycle.value == 1 then player.baudrate = 10
							else player.baudrate = 0 end
						end
					else
						local file = P_SpawnPlayerMissile(player.mo, MT_WINFILE, (MF2_EXPLOSION | MF2_SUPERFIRE))
						if player.powers[pw_super] != 0 then
							if player.hyper and player.hyper.capable then
								player.baudrate = 18
								file.momx = $*3/2
								file.momy = $*3/2
								file.momz = $*3/2
							elseif intenserecycle.value == 1 then
								player.baudrate = 18
								file.momx = $*4/3
								file.momy = $*4/3
								file.momz = $*4/3
							else
								player.baudrate = 14
								file.momx = $*4/3
								file.momy = $*4/3
								file.momz = $*4/3
							end
						else
							if intenserecycle.value == 1 then player.baudrate = 10
							else player.baudrate = 0 end
						end
					end
				end
			end
		else
			// No shooting willy nilly if you already are using Rings as ammo.
			// You'd have to have someone between you and the monitor to hit them
			// with a file instead. Not likely, but it might happen anyway.
			if G_RingSlingerGametype() or G_TagGametype() then return end
			if player.baudrate >= 19 and P_PlayerInPain(player) == false and player.playerstate != PST_DEAD and player.exiting == 0 then
				if player.filetossfire then
					local file = P_SpawnPlayerMissile(player.mo, MT_WINFILE, (MF2_EXPLOSION | MF2_SUPERFIRE))
					if player.powers[pw_super] != 0 then
						if player.hyper and player.hyper.capable then
							player.baudrate = 18
							file.momx = $*3/2
							file.momy = $*3/2
							file.momz = $*3/2
						elseif intenserecycle.value == 1 then
							player.baudrate = 18
							file.momx = $*4/3
							file.momy = $*4/3
							file.momz = $*4/3
						else
							player.baudrate = 14
							file.momx = $*4/3
							file.momy = $*4/3
							file.momz = $*4/3
						end
					else
						if intenserecycle.value == 1 then player.baudrate = 10
						else player.baudrate = 0 end
					end
				end
			end
		end
	end
end)